@@ -119,11 +119,11 @@ MainWindow::MainWindow()
     setupToolsPluginsAction();
     setupDSPluginsAction();
 
-    connect(DocumentManager::self(), SIGNAL(activateDocument()),
+    connect(&DocumentManager::self(), SIGNAL(activateDocument()),
             this, SLOT(setActiveDocument()));
-    connect(DocumentManager::self(), SIGNAL(deactivateDocument(Document*)),
+    connect(&DocumentManager::self(), SIGNAL(deactivateDocument(Document*)),
             this, SLOT(releaseDocument(Document*)));
-    connect(DocumentManager::self(), SIGNAL(documentRemoved(Document*)),
+    connect(&DocumentManager::self(), SIGNAL(documentRemoved(Document*)),
             this, SLOT(releaseDocument(Document*)));
 
     // TODO: use welcome widget instead of creating default empty project
@@ -221,9 +221,10 @@ void MainWindow::setupToolbars()
 
 Project* MainWindow::createNewProject()
 {
-    Project* newProject = new Project();
+    //TODO duplicated functionality with newProject(): merge them for 4.11
+    Project *newProject = new Project();
     // create new document and add this to project new
-    newProject->addGraphFileNew(DocumentManager::self()->newDocument());
+    newProject->addGraphFileNew(DocumentManager::self().newDocument());
     newProject->addCodeFileNew(_codeEditor->newScript());
     _journalWidget->openJournal(newProject);
 
@@ -232,8 +233,8 @@ Project* MainWindow::createNewProject()
 
 void MainWindow::downloadNewExamples()
 {
-    KNS3::DownloadDialog dialog("rocs.knsrc", this);
-    dialog.exec();
+    QPointer<KNS3::DownloadDialog> dialog = new KNS3::DownloadDialog("rocs.knsrc", this);
+    dialog->exec();
 }
 
 void MainWindow::uploadScript()
@@ -340,7 +341,7 @@ QWidget* MainWindow::setupSidePanel()
     // add widgets to dock
     // document property widgets
     DocumentTypesWidget* documentTypesWidget = new DocumentTypesWidget(this);
-    connect(DocumentManager::self(), SIGNAL(activateDocument()), documentTypesWidget, SLOT(updateDocument()));
+    connect(&DocumentManager::self(), SIGNAL(activateDocument()), documentTypesWidget, SLOT(updateDocument()));
     sideDock->addDock(documentTypesWidget, i18n("Element Types"), KIcon("document-properties"));
 
     // Project Journal
@@ -414,32 +415,32 @@ void MainWindow::createAction(const QByteArray& iconName, const QString& actionT
 
 void MainWindow::showSettings()
 {
-    KConfigDialog dialog(this,  "settings", Settings::self());
+    QPointer<KConfigDialog> dialog = new KConfigDialog(this,  "settings", Settings::self());
 
-    IncludeManagerSettings * set = new IncludeManagerSettings(&dialog);
-    ConfigureDefaultProperties * defaultProperties = new ConfigureDefaultProperties(&dialog);
+    IncludeManagerSettings * set = new IncludeManagerSettings(dialog);
+    ConfigureDefaultProperties * defaultProperties = new ConfigureDefaultProperties(dialog);
 
-    dialog.addPage(set, i18nc("@title:tab", "Include Manager"), QString(), i18nc("@title:tab", "Include Manager"), true);
-    dialog.addPage(defaultProperties, i18nc("@title:tab", "Default Settings"), QString(), i18nc("@title:tab", "Default Settings"), true);
+    dialog->addPage(set, i18nc("@title:tab", "Include Manager"), QString(), i18nc("@title:tab", "Include Manager"), true);
+    dialog->addPage(defaultProperties, i18nc("@title:tab", "Default Settings"), QString(), i18nc("@title:tab", "Default Settings"), true);
 
 
-    connect(set,               SIGNAL(changed(bool)), &dialog, SLOT(enableButtonApply(bool)));
-    connect(defaultProperties, SIGNAL(changed(bool)), &dialog, SLOT(enableButtonApply(bool)));
+    connect(set,               SIGNAL(changed(bool)), dialog, SLOT(enableButtonApply(bool)));
+    connect(defaultProperties, SIGNAL(changed(bool)), dialog, SLOT(enableButtonApply(bool)));
 
-    connect(&dialog, SIGNAL(applyClicked()),   set, SLOT(saveSettings()));
-    connect(&dialog, SIGNAL(okClicked()),      set, SLOT(saveSettings()));
-    connect(&dialog, SIGNAL(defaultClicked()), set, SLOT(readConfig()));
+    connect(dialog, SIGNAL(applyClicked()),   set, SLOT(saveSettings()));
+    connect(dialog, SIGNAL(okClicked()),      set, SLOT(saveSettings()));
+    connect(dialog, SIGNAL(defaultClicked()), set, SLOT(readConfig()));
 
-    connect(&dialog, SIGNAL(applyClicked()),   defaultProperties, SLOT(saveConfig()));
-    connect(&dialog, SIGNAL(okClicked()),      defaultProperties, SLOT(saveConfig()));
-    connect(&dialog, SIGNAL(defaultClicked()), defaultProperties, SLOT(readConfig()));
+    connect(dialog, SIGNAL(applyClicked()),   defaultProperties, SLOT(saveConfig()));
+    connect(dialog, SIGNAL(okClicked()),      defaultProperties, SLOT(saveConfig()));
+    connect(dialog, SIGNAL(defaultClicked()), defaultProperties, SLOT(readConfig()));
 
     connect(defaultProperties, SIGNAL(showExecuteModeDebugChanged(bool)),
             this, SLOT(showExecutionButtonDebug(bool)));
     connect(defaultProperties, SIGNAL(showExecuteModeOneStepChanged(bool)),
             this, SLOT(showExecutionButtonOneStep(bool)));
 
-    dialog.exec();
+    dialog->exec();
 }
 
 void MainWindow::setupToolsPluginsAction()
@@ -451,8 +452,8 @@ void MainWindow::setupToolsPluginsAction()
     foreach(QAction* action, _toolsPlugins) {
         ToolsPluginInterface *plugin = ToolManager::instance()->plugins().at(action->data().toInt());
         action->setEnabled(
-                DocumentManager::self()->activeDocument() &&
-                plugin->supportedDataStructures().contains(DocumentManager::self()->activeDocument()->dataStructureInternalName())
+                DocumentManager::self().activeDocument() &&
+                plugin->supportedDataStructures().contains(DocumentManager::self().activeDocument()->backend()->internalName())
                           );
     }
 }
@@ -508,15 +509,15 @@ void MainWindow::setupDocumentsList()
     unplugActionList("Doc_List");
     QActionGroup * group = new QActionGroup(this);
     int count = 0;
-    foreach(Document * doc, DocumentManager::self()->documentList()) {
+    foreach(Document * doc, DocumentManager::self().documentList()) {
         action = new KAction(doc->name(), this);
         action->setData(count++);
         action->setCheckable(true);
-        if (doc == DocumentManager::self()->activeDocument()) {
+        if (doc == DocumentManager::self().activeDocument()) {
             action->setChecked(true);
         }
         action->setActionGroup(group);
-        connect(action, SIGNAL(triggered(bool)), DocumentManager::self(), SLOT(changeDocument()));
+        connect(action, SIGNAL(triggered(bool)), &DocumentManager::self(), SLOT(changeDocument()));
         pluginList.append(action);
     }
 
@@ -526,7 +527,7 @@ void MainWindow::setupDocumentsList()
 void MainWindow::setActiveDocument()
 {
     kDebug() << "Setting the document in the main window";
-    Document *activeDocument = DocumentManager::self()->activeDocument();
+    Document *activeDocument = DocumentManager::self().activeDocument();
     QtScriptBackend *engine = activeDocument->engineBackend();
 
     _graphVisualEditor->setActiveDocument();
@@ -548,14 +549,15 @@ void MainWindow::releaseDocument(Document* d)
     d->disconnect(this);
     disconnect(d);
 
-    _graphVisualEditor->releaseDocument();
+    d->engineBackend()->stop();
     d->engineBackend()->disconnect(this);
+    _graphVisualEditor->releaseDocument();
 }
 
 void MainWindow::addEmptyGraphDocument()
 {
     _currentProject->addGraphFileNew(
-        DocumentManager::self()->newDocument()
+        DocumentManager::self().newDocument()
     );
 }
 
@@ -587,22 +589,23 @@ void MainWindow::loadDocument(const QString& name)
         return;
     }
 
-    DocumentManager::self()->openDocument(KUrl::fromLocalFile(name));
+    DocumentManager::self().openDocument(KUrl::fromLocalFile(name));
 }
 
 
 void MainWindow::newProject()
 {
-    if (saveIfChanged() == KMessageBox::Cancel)
+    if (saveIfChanged() == KMessageBox::Cancel) {
         return;
+    }
 
     _codeEditor->closeAllScripts();
-    DocumentManager::self()->closeAllDocuments();
+    DocumentManager::self().closeAllDocuments();
 
     delete _currentProject;
     _currentProject = new Project();
     _currentProject->addCodeFileNew(_codeEditor->newScript());
-    _currentProject->addGraphFileNew(DocumentManager::self()->newDocument());
+    _currentProject->addGraphFileNew(DocumentManager::self().newDocument());
     _journalWidget->openJournal(_currentProject);
     updateCaption();
 }
@@ -664,8 +667,9 @@ void MainWindow::saveProjectAs()
 
 void MainWindow::openProject(const KUrl& fileName)
 {
-    if (saveIfChanged() == KMessageBox::Cancel)
+    if (saveIfChanged() == KMessageBox::Cancel) {
         return;
+    }
 
     KUrl startDirectory = Settings::lastOpenedDirectory();
     KUrl file = fileName;
@@ -681,8 +685,9 @@ void MainWindow::openProject(const KUrl& fileName)
         }
     }
     // import project specified: close everything and delete old project
+    DocumentManager::self().activeDocument()->engineBackend()->stop();
+    DocumentManager::self().closeAllDocuments();
     _codeEditor->closeAllScripts();
-    DocumentManager::self()->closeAllDocuments();
     delete _currentProject;
 
     // extract and open new project
@@ -690,16 +695,16 @@ void MainWindow::openProject(const KUrl& fileName)
     if (file.fileName().endsWith(QLatin1String("rocsz"), Qt::CaseInsensitive)){
         _currentProject = new Project(file, file.directory(KUrl::AppendTrailingSlash));
         foreach(const KUrl& graphFile, _currentProject->graphFiles()) {
-            DocumentManager::self()->openDocument(graphFile);
+            DocumentManager::self().openDocument(graphFile);
         }
     } else {
         _currentProject = new Project(file);
         foreach(const KUrl& graphFile, _currentProject->graphFiles()) {
-            DocumentManager::self()->openDocument(graphFile);
+            DocumentManager::self().openDocument(graphFile);
         }
     }
     if (_currentProject->graphFiles().count() == 0) {
-        _currentProject->addGraphFileNew(DocumentManager::self()->newDocument());
+        _currentProject->addGraphFileNew(DocumentManager::self().newDocument());
     }
     foreach(const KUrl& codeFile, _currentProject->codeFiles()) {
         _codeEditor->openScript(codeFile);
@@ -764,7 +769,7 @@ void MainWindow::newScript()
 void MainWindow::saveGraph(Document* document)
 {
     if (document == 0) {
-        document = DocumentManager::self()->activeDocument();
+        document = DocumentManager::self().activeDocument();
     }
     Q_ASSERT(document);
     if (document->fileUrl().isEmpty()) {
@@ -777,7 +782,7 @@ void MainWindow::saveGraph(Document* document)
 
 void MainWindow::saveAllGraphs()
 {
-    foreach(Document * document, DocumentManager::self()->documentList()) {
+    foreach(Document * document, DocumentManager::self().documentList()) {
         if (document->fileUrl().isEmpty()) {
             _currentProject->saveGraphFileAs(document,
                                              _currentProject->projectFile().toLocalFile().remove(QRegExp(".rocsz*$"))
@@ -791,14 +796,14 @@ void MainWindow::saveAllGraphs()
 
 void MainWindow::saveGraphAs()
 {
-    saveGraphAs(DocumentManager::self()->activeDocument());
+    saveGraphAs(DocumentManager::self().activeDocument());
 }
 
 
 void MainWindow::saveGraphAs(Document* document)
 {
     if (document == 0) {
-        document = DocumentManager::self()->activeDocument();
+        document = DocumentManager::self().activeDocument();
     }
     Q_ASSERT(document);
     QString file = KFileDialog::getSaveFileName(QString(),
@@ -819,7 +824,7 @@ void MainWindow::newGraph()
     if (!file.endsWith(QLatin1String(".graph"))){
         file.append(".graph");
     }
-    DocumentManager::self()->openDocument(KUrl::fromLocalFile(file));
+    DocumentManager::self().openDocument(KUrl::fromLocalFile(file));
     _currentProject->addGraphFile(file);
 }
 
@@ -838,7 +843,7 @@ int MainWindow::saveIfChanged()
     }
 
     bool anyGraphDocumentModified = false;
-    foreach(Document * document, DocumentManager::self()->documentList()) {
+    foreach(Document * document, DocumentManager::self().documentList()) {
         if (document->isModified()) {
             anyGraphDocumentModified = true;
             break;
@@ -853,7 +858,7 @@ int MainWindow::saveIfChanged()
         }
         return btnCode;
     }
-    if (!DocumentManager::self()->activeDocument()->isModified() && _codeEditor->isModified()) {
+    if (!DocumentManager::self().activeDocument()->isModified() && _codeEditor->isModified()) {
         const int btnCode = KMessageBox::warningYesNoCancel(this, i18nc(
                                 "@info",
                                 "Changes on your script files are unsaved. Do you want to save all unsaved scripts?"));
@@ -883,7 +888,7 @@ void MainWindow::importGraphFile()
         return;
     }
 
-    DocumentManager::self()->addDocument(gd);
+    DocumentManager::self().addDocument(gd);
 
     if (importer.hasDialog()) {
         importer.dialogExec();
@@ -895,26 +900,26 @@ void MainWindow::exportGraphFile()
 {
     ImporterExporterManager exp(this);
 
-    exp.exportFile(DocumentManager::self()->activeDocument());
+    exp.exportFile(DocumentManager::self().activeDocument());
 }
 
 void MainWindow::showPossibleIncludes()
 {
-    PossibleIncludes dialog(this);
-    dialog.exec();
+    QPointer<PossibleIncludes> dialog = new PossibleIncludes(this);
+    dialog->exec();
 }
 
 
 void MainWindow::showLoadedPlugins()
 {
-    LoadedPluginsDialog dialog(this);
-    dialog.exec();
+    QPointer<LoadedPluginsDialog> dialog = new LoadedPluginsDialog(this);
+    dialog->exec();
 }
 
 
 void MainWindow::showCodeEditorConfig()
 {
-    KTextEditor::Editor* editor = _codeEditor->editor();
+    KTextEditor::Editor *editor = _codeEditor->editor();
     if (editor->configDialogSupported()) {
         editor->configDialog(this);
         editor->writeConfig();
@@ -930,7 +935,7 @@ void MainWindow::runToolPlugin()
         return;
     }
     if (ToolsPluginInterface *plugin =  ToolManager::instance()->plugins().value(action->data().toInt())) {
-        plugin->run(DocumentManager::self()->activeDocument());
+        plugin->run(DocumentManager::self().activeDocument());
     }
 }
 
@@ -952,7 +957,7 @@ void MainWindow::executeScript(const MainWindow::ScriptMode mode, const QString&
 
     QString script = text.isEmpty() ? _codeEditor->text() : text;
     QString scriptPath = _codeEditor->document()->url().path();
-    QtScriptBackend *engine = DocumentManager::self()->activeDocument()->engineBackend();
+    QtScriptBackend *engine = DocumentManager::self().activeDocument()->engineBackend();
     if (engine->isRunning()) {
         engine->stop();
     }
@@ -976,7 +981,7 @@ void MainWindow::executeScript(const MainWindow::ScriptMode mode, const QString&
 
     enableStopAction();
 
-    engine->setScript(script, DocumentManager::self()->activeDocument());
+    engine->setScript(script, DocumentManager::self().activeDocument());
     engine->execute();
 }
 
@@ -985,7 +990,7 @@ void MainWindow::executeScriptOneStep(const QString& text)
 {
     Q_ASSERT(_outputWidget);
 
-    QtScriptBackend *engine = DocumentManager::self()->activeDocument()->engineBackend();
+    QtScriptBackend *engine = DocumentManager::self().activeDocument()->engineBackend();
 
     //TODO disable start action
     enableStopAction();
@@ -1001,7 +1006,7 @@ void MainWindow::executeScriptOneStep(const QString& text)
                              scriptPath.isEmpty() ? scriptPath : scriptPath.section('/', 0, -2),
                              _codeEditor->document()->documentName());
 
-        engine->setScript(script, DocumentManager::self()->activeDocument());
+        engine->setScript(script, DocumentManager::self().activeDocument());
         engine->executeStep();
         return;
     }
@@ -1010,7 +1015,7 @@ void MainWindow::executeScriptOneStep(const QString& text)
 
 void MainWindow::stopScript()
 {
-    QtScriptBackend *engine = DocumentManager::self()->activeDocument()->engineBackend();
+    QtScriptBackend *engine = DocumentManager::self().activeDocument()->engineBackend();
     disableStopAction();
     engine->stop();
 }
